//ArcBall.cpp
#include <windows.h>
#include <time.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <stdio.h>

#include "ArcBall.h"

ArcBall::ArcBall(void)
: mIsDragged(false)
, mRadius(1.0f)
, mPreviousPoint(D3DXVECTOR3(0, 0, 0))
, mCurrentPoint(D3DXVECTOR3(0, 0, 0))
, mOldPoint(D3DXVECTOR3(0, 0, 0))
, mPreviousQuaternion(D3DXQUATERNION(0, 0, 0, 1))
, mCurrentQuaternion(D3DXQUATERNION(0, 0, 0, 1))
, mRotationIncreament(D3DXQUATERNION(0, 0, 0, 1))
{
	D3DXMatrixIdentity(&mRotateMat);

	RECT rc ;
	GetClientRect(GetForegroundWindow(), &rc);

	int windowWidth  = rc.right - rc.left;
	int windowHeight = rc.bottom - rc.top;

	SetWindow(windowWidth, windowHeight);
}

ArcBall::~ArcBall(void)
{
}

void ArcBall::Reset()
{
	D3DXQuaternionIdentity(&mPreviousQuaternion);
	D3DXQuaternionIdentity(&mCurrentQuaternion);
	D3DXQuaternionIdentity(&mRotationIncreament);
	D3DXMatrixIdentity(&mRotateMat);
	mIsDragged = false;
	mRadius = 1.0f;
}

void ArcBall::OnBegin(int mousex, int mousey)
{
	// enter drag state only if user click the window's client area
	if(mousex >= 0 && mousex <= mWindowWidth 
	   && mousey >= 0 && mousey < mWindowHeight)
	{
		mIsDragged = true ; // begin drag state
		mPreviousQuaternion = mCurrentQuaternion;
		mPreviousPoint = ScreenToVector(mousex, mousey);
		mOldPoint = mPreviousPoint;
	}
}

void ArcBall::OnMove(int mousex, int mousey)
{
	if(mIsDragged)
	{
		mCurrentPoint = ScreenToVector(mousex, mousey);
		mRotationIncreament = QuatFromBallPoints( mOldPoint, mCurrentPoint);
		mCurrentQuaternion = mPreviousQuaternion * QuatFromBallPoints( mPreviousPoint, mCurrentPoint );
		mOldPoint = mCurrentPoint;
	}
}

void ArcBall::OnEnd()
{
	mIsDragged = false;
}

void ArcBall::SetWindow(int windowWidth, int windowHeight, float arcballRadius)
{
	 mWindowWidth  = windowWidth; 
	 mWindowHeight = windowHeight; 
	 mRadius = arcballRadius; 
}

const D3DXMATRIX* ArcBall::GetRotationMatrix()
{
	return D3DXMatrixRotationQuaternion(&mRotateMat, &mCurrentQuaternion);
}

D3DXQUATERNION ArcBall::GetRotationQuatIncreament()
{
	return mRotationIncreament;
}

D3DXQUATERNION ArcBall::QuatFromBallPoints(D3DXVECTOR3& startPoint, D3DXVECTOR3& endPoint)
{
	// Calculate rotate angle
	float angle = D3DXVec3Dot(&startPoint, &endPoint);	

	// Calculate rotate axis
	D3DXVECTOR3 axis;
	D3DXVec3Cross(&axis, &startPoint, &endPoint);		

	// Build and Normalize the Quaternion
	D3DXQUATERNION quat(axis.x, axis.y, axis.z, angle);
	D3DXQuaternionNormalize(&quat, &quat);

	return quat;
}

D3DXVECTOR3 ArcBall::ScreenToVector(int fScreenPtX, int fScreenPtY)
{
	// Scale to screen
	float x = -(fScreenPtX - mWindowWidth / 2) / (mRadius * mWindowWidth / 2);
	float y = (fScreenPtY - mWindowHeight / 2) / (mRadius * mWindowHeight / 2);

	float z = 0.0f;
	float mag = x * x + y * y;

	if(mag > 1.0f)
	{
		float scale = 1.0f / sqrtf(mag);
		x *= scale;
		y *= scale;
	}
	else
		z = sqrtf(1.0f - mag);

	return D3DXVECTOR3(x, y, z);
}
