//ArcBallCamera.h
#ifndef _ARCBALLCAMERA_H
#define _ARCBALLCAMERA_H

#include "ArcBall.h"

class ArcBallCamera
{
public:
	ArcBallCamera(void);
	~ArcBallCamera(void);

public:
	void Reset();
	void ForceRotate();
	void OnFrameMove();
	LRESULT HandleMessages(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	void SetViewParams(const D3DXVECTOR3& eypePos, const D3DXVECTOR3& lookatPos, const D3DXVECTOR3& upVector);
	void SetProjParams(float fieldOfView, float aspectRatio, float nearPlane, float farPlane) ;
	void SetWindow(int windowWidth, int windowHeight, float arcballRadius = 1.0f);
	const D3DXMATRIX GetWorldMatrix() const;
	const D3DXMATRIX GetViewMatrix() const;
	const D3DXMATRIX GetProjMatrix() const;
	const D3DXVECTOR3 GetEyePoint() const;

private:
	bool	mNeedToUpdateFrame;
	float	mRadius;				// Distance from the camera to model 
	float	mMaxRadius;			// The Maximum distance from the camera to the model
	float	mMinRadius;			// The Minimum distance from the camera to the model
	int		mMouseWheelDelta;		// Amount of middle wheel scroll (+/-)
	
	D3DXVECTOR3 mEyePos;			// Eye position
	D3DXVECTOR3 mLookatPos;		// Look at position
	D3DXVECTOR3 mUpVector;				// Up vector

	D3DXMATRIX mWorldMat;			// World matrix of model
	D3DXMATRIX mViewMat;			// ArcBallCamera View matrix
	D3DXMATRIX mProjMat;			// ArcBallCamera Projection matrix

	ArcBall mViewArcBall;			// View arc ball
};

#endif 